# -*- coding: utf-8 -*-
require 'test_helper'
require 'integration_test_helper'

class NestedHasManyTest < ActionDispatch::IntegrationTest
  self.use_transactional_fixtures = false

  setup do
    DatabaseCleaner.start
    @admin = create(:admin)
    @verify_list = []
    @discussion = create(:story_status, :name => "discussion")
    @implementation = create(:story_status, :name => "implementation")
    @project = create(:project, :name => "First Project", :owner => @admin)
    @s1 = create(:story, :project => @project, :title => "First Story", :body => "First Story", :status => @discussion)
    @s2 = create(:story, :project => @project, :title => "Second story for first project", :body => "Second story for first project (body)", :status => nil)
    @t1 = create(:task, :story => @s1, :description => "First Task", :position => 1)
  end

  teardown do
    DatabaseCleaner.clean
  end

  test "nested has_many" do
    visit root_path

    # Resize the window so Bootstrap shows Login button
    Capybara.current_session.driver.resize(1024,700)

    # log in as Administrator
    click_link "Log out" rescue Capybara::ElementNotFound
    click_link "Login"
    fill_in "login", :with => "admin@example.com"
    fill_in "password", :with => "test123"
    click_button "Login"
    assert has_content?("Logged in as Admin User")

    visit "/projects/#{@project.id}/nested_has_many_test"

    # second story has no tasks but minimal="1", so should have empty task
    assert_equal find("#project_stories_1_tasks_0_description").value, ""
    assert page.has_selector?('#project_stories_1_tasks_0_add', visible: true)
    assert page.has_selector?('#project_stories_1_tasks_0_remove', visible: false)

    # first story only has a single task
    assert page.has_selector?('#project_stories_0_tasks_0_add', visible: true)
    assert page.has_selector?('#project_stories_0_tasks_0_remove', visible: false)

    # verify button customized
    assert find("#project_stories_0_remove").has_text?("Remove Story")

    # verify fields customized
    assert has_no_css?("#project_stories_0_tasks_0_position")

    # verify add inner
    click_button "project_stories_0_tasks_0_add"
    assert has_field?("project_stories_0_tasks_1_description")
    fill_in "project_stories_0_tasks_1_description", :with => "Second task for first story"
    assert page.has_selector?('#project_stories_0_tasks_0_add', visible: false)
    assert find("#project_stories_0_tasks_0_remove").visible?
    assert find("#project_stories_0_tasks_1_add").visible?
    assert find("#project_stories_0_tasks_1_remove").visible?

    click_button "project_stories_0_tasks_1_add"
    assert has_field?("project_stories_0_tasks_2_description")
    fill_in "project_stories_0_tasks_2_description", :with => "Third task for first story"
    assert page.has_selector?('#project_stories_0_tasks_1_add', visible: false)
    assert find("#project_stories_0_tasks_1_remove").visible?
    assert find("#project_stories_0_tasks_2_add").visible?
    assert find("#project_stories_0_tasks_2_remove").visible?

    # verify save
    click_button "Save"
    assert find(".table-plus tbody tr:nth-child(1) td.tasks-count-view span").has_text?("3")
    visit "/projects/#{@project.id}/nested_has_many_test"

    # verify remove inner
    click_button "project_stories_0_tasks_1_remove"
    assert has_no_field?("project_stories_0_tasks_2_description")
    assert_equal find("#project_stories_0_tasks_1_description").value, "Third task for first story"
    assert page.has_selector?('#project_stories_0_tasks_0_add', visible: false)
    assert find("#project_stories_0_tasks_0_remove").visible?
    assert find("#project_stories_0_tasks_1_add").visible?
    assert find("#project_stories_0_tasks_1_remove").visible?

    click_button "project_stories_0_tasks_0_remove"
    assert has_no_field?("project_stories_0_tasks_1_description")
    assert_equal find("#project_stories_0_tasks_0_description").value, "Third task for first story"
    assert find("#project_stories_0_tasks_0_add").visible?
    assert page.has_selector?('#project_stories_0_tasks_0_remove', visible: false)

    # verify add outer
    click_button "project_stories_1_add"
    assert has_field?("project_stories_2_tasks_0_description")
    assert_equal find("#project_stories_2_tasks_0_description").value, ""
    assert find("#project_stories_2_tasks_0_add").visible?
    assert page.has_selector?('#project_stories_2_tasks_0_remove', visible: false)
    click_button "project_stories_2_tasks_0_add"
    assert page.has_selector?('#project_stories_2_tasks_0_add', visible: false)
    assert find("#project_stories_2_tasks_0_remove").visible?
    assert find("#project_stories_2_tasks_1_add").visible?
    assert find("#project_stories_2_tasks_1_remove").visible?
    fill_in "project_stories_2_tasks_1_description", :with => "Third story Second task"

    #verify remove outer
    click_button "project_stories_0_remove"
    assert has_no_field?("project_stories_2_title")
    assert_equal find("#project_stories_0_title").value, "Second story for first project"

    click_button "project_stories_0_remove"
    assert has_no_field?("project_stories_1_title")
    assert_equal find("#project_stories_0_tasks_1_description").value, "Third story Second task"

    click_button "project_stories_0_remove"
    assert has_no_field?("project_stories_0_title")
    assert has_no_css?("#project_stories_0_tasks_1_description")

    assert find("#project_stories_-1_empty").visible?

  end
end
